from typing import Any

from django.db import models
from django.db.models.base import Model
from django.db.models.query import QuerySet

class ContentTypeManager(models.Manager[ContentType]):
    def get_by_natural_key(self, app_label: str, model: str) -> ContentType: ...
    def get_for_model(self, model: type[Model] | Model, for_concrete_model: bool = ...) -> ContentType: ...
    def get_for_models(self, *models: Any, for_concrete_models: bool = ...) -> dict[type[Model], ContentType]: ...
    def get_for_id(self, id: int) -> ContentType: ...
    def clear_cache(self) -> None: ...

class ContentType(models.Model):
    id: int
    app_label: models.CharField
    model: models.CharField
    objects: ContentTypeManager
    @property
    def name(self) -> str: ...
    def model_class(self) -> type[Model] | None: ...
    def get_object_for_this_type(self, **kwargs: Any) -> Model: ...
    def get_all_objects_for_this_type(self, **kwargs: Any) -> QuerySet: ...
    def natural_key(self) -> tuple[str, str]: ...



from collections.abc import Callable
from typing import TypeVar
from typing_extensions import ParamSpec

_T = TypeVar("_T")
_P = ParamSpec("_P")

def _clear() -> None: ...
def _ncallbacks() -> int: ...
def _run_exitfuncs() -> None: ...
def register(func: Callable[_P, _T], *args: _P.args, **kwargs: _P.kwargs) -> Callable[_P, _T]: ...
def unregister(func: Callable[..., object]) -> None: ...



from collections.abc import Sequence
from typing import Any

from django.db.migrations.migration import Migration, SwappableTuple
from django.db.migrations.state import ProjectState

RECURSION_DEPTH_WARNING: str

class Node:
    key: tuple[str, str]
    children: set[Any]
    parents: set[Any]
    def __init__(self, key: tuple[str, str]) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __lt__(self, other: tuple[str, str] | Node) -> bool: ...
    def __getitem__(self, item: int) -> str: ...
    def __hash__(self) -> int: ...
    def add_child(self, child: Node) -> None: ...
    def add_parent(self, parent: Node) -> None: ...

class DummyNode(Node):
    origin: Any
    error_message: Any
    def __init__(self, key: tuple[str, str], origin: Migration | str, error_message: str) -> None: ...
    def raise_error(self) -> None: ...

class MigrationGraph:
    node_map: dict[tuple[str, str], Node]
    nodes: dict[tuple[str, str], Migration | None]
    cached: bool
    def __init__(self) -> None: ...
    def add_node(self, key: tuple[str, str], migration: Migration | None) -> None: ...
    def add_dummy_node(self, key: tuple[str, str], origin: Migration | str, error_message: str) -> None: ...
    def add_dependency(
        self,
        migration: Migration | str | None,
        child: tuple[str, str],
        parent: tuple[str, str],
        skip_validation: bool = ...,
    ) -> None: ...
    def remove_replaced_nodes(self, replacement: tuple[str, str], replaced: list[tuple[str, str]]) -> None: ...
    def remove_replacement_node(self, replacement: tuple[str, str], replaced: list[tuple[str, str]]) -> None: ...
    def validate_consistency(self) -> None: ...
    def forwards_plan(self, target: tuple[str, str]) -> list[tuple[str, str]]: ...
    def backwards_plan(self, target: tuple[str, str]) -> list[tuple[str, str]]: ...
    def iterative_dfs(self, start: Any, forwards: bool = ...) -> list[tuple[str, str]]: ...
    def root_nodes(self, app: str | None = ...) -> list[tuple[str, str]]: ...
    def leaf_nodes(self, app: str | None = ...) -> list[tuple[str, str]]: ...
    def ensure_not_cyclic(self) -> None: ...
    def make_state(
        self,
        nodes: None | tuple[str, str] | Sequence[tuple[str, str]] = ...,
        at_end: bool = ...,
        real_apps: list[str] = ...,
    ) -> ProjectState: ...
    def __contains__(self, node: tuple[str, str]) -> bool: ...



from .base import RedirectView as RedirectView
from .base import TemplateView as TemplateView
from .base import View as View
from .dates import ArchiveIndexView as ArchiveIndexView
from .dates import DateDetailView as DateDetailView
from .dates import DayArchiveView as DayArchiveView
from .dates import MonthArchiveView as MonthArchiveView
from .dates import TodayArchiveView as TodayArchiveView
from .dates import WeekArchiveView as WeekArchiveView
from .dates import YearArchiveView as YearArchiveView
from .detail import DetailView as DetailView
from .edit import CreateView as CreateView
from .edit import DeleteView as DeleteView
from .edit import FormView as FormView
from .edit import UpdateView as UpdateView
from .list import ListView as ListView

class GenericViewError(Exception): ...



from collections.abc import Iterable
from html.parser import HTMLParser
from re import Pattern
from typing import Any

from django.utils.safestring import SafeString

TRAILING_PUNCTUATION_CHARS: str
WRAPPING_PUNCTUATION: list[tuple[str, str]]
DOTS: list[str]
word_split_re: Pattern[str]
simple_url_re: Pattern[str]
simple_url_2_re: Pattern[str]

def escape(text: Any) -> SafeString: ...
def escapejs(value: Any) -> SafeString: ...
def json_script(value: Any, element_id: str) -> SafeString: ...
def conditional_escape(text: Any) -> str: ...
def format_html(format_string: str, *args: Any, **kwargs: Any) -> SafeString: ...
def format_html_join(sep: str, format_string: str, args_generator: Iterable[Iterable[Any]]) -> SafeString: ...
def linebreaks(value: Any, autoescape: bool = ...) -> str: ...

class MLStripper(HTMLParser):
    fed: Any
    def __init__(self) -> None: ...
    def handle_data(self, d: str) -> None: ...
    def handle_entityref(self, name: str) -> None: ...
    def handle_charref(self, name: str) -> None: ...
    def get_data(self) -> str: ...

def strip_tags(value: str) -> str: ...
def strip_spaces_between_tags(value: str) -> str: ...
def smart_urlquote(url: str) -> str: ...
def urlize(text: str, trim_url_limit: int | None = ..., nofollow: bool = ..., autoescape: bool = ...) -> str: ...
def avoid_wrapping(value: str) -> str: ...
def html_safe(klass: type) -> type: ...



from typing import Any

from django.db.backends.base.introspection import BaseDatabaseIntrospection
from django.db.backends.sqlite3.base import DatabaseWrapper

field_size_re: Any

def get_field_size(name: str) -> int | None: ...

class FlexibleFieldLookupDict:
    base_data_types_reverse: Any
    def __getitem__(self, key: str) -> Any: ...

class DatabaseIntrospection(BaseDatabaseIntrospection):
    connection: DatabaseWrapper



import functools
from collections.abc import Callable, Iterator, Sequence
from http.cookies import SimpleCookie
from typing import Any, Union  # noqa: Y037   # https://github.com/python/mypy/issues/12211

from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse
from django.http.request import HttpRequest
from django.template.base import Template
from django.template.context import RequestContext
from django.test.client import Client
from django.utils.datastructures import _ListOrTuple
from typing_extensions import TypeAlias

# https://github.com/python/mypy/issues/12211
_TemplateForResponseT: TypeAlias = Union[_ListOrTuple[str], Template, str]

class ContentNotRenderedError(Exception): ...

class SimpleTemplateResponse(HttpResponse):
    content: Any
    closed: bool
    cookies: SimpleCookie[str]
    status_code: int
    rendering_attrs: Any
    template_name: _TemplateForResponseT
    context_data: dict[str, Any] | None
    using: str | None
    def __init__(
        self,
        template: _TemplateForResponseT,
        context: dict[str, Any] | None = ...,
        content_type: str | None = ...,
        status: int | None = ...,
        charset: str | None = ...,
        using: str | None = ...,
        headers: dict[str, Any] | None = ...,
    ) -> None: ...
    def resolve_template(self, template: Sequence[str] | Template | str) -> Template: ...
    def resolve_context(self, context: dict[str, Any] | None) -> dict[str, Any] | None: ...
    @property
    def rendered_content(self) -> str: ...
    def add_post_render_callback(self, callback: Callable) -> None: ...
    def render(self) -> SimpleTemplateResponse: ...
    @property
    def is_rendered(self) -> bool: ...
    def __iter__(self) -> Iterator[Any]: ...

class TemplateResponse(SimpleTemplateResponse):
    client: Client
    closed: bool
    context: RequestContext
    context_data: dict[str, Any] | None
    cookies: SimpleCookie[str]
    csrf_cookie_set: bool
    json: functools.partial
    _request: HttpRequest
    status_code: int
    template_name: _TemplateForResponseT
    templates: list[Template]
    using: str | None
    wsgi_request: WSGIRequest
    rendering_attrs: Any
    def __init__(
        self,
        request: HttpRequest,
        template: _TemplateForResponseT,
        context: dict[str, Any] | None = ...,
        content_type: str | None = ...,
        status: int | None = ...,
        charset: str | None = ...,
        using: str | None = ...,
        headers: dict[str, Any] | None = ...,
    ) -> None: ...



import sys
import types
import unittest
from typing import Any, Callable, Dict, List, NamedTuple, Optional, Tuple, Type, Union

class TestResults(NamedTuple):
    failed: int
    attempted: int

OPTIONFLAGS_BY_NAME: Dict[str, int]

def register_optionflag(name: str) -> int: ...

DONT_ACCEPT_TRUE_FOR_1: int
DONT_ACCEPT_BLANKLINE: int
NORMALIZE_WHITESPACE: int
ELLIPSIS: int
SKIP: int
IGNORE_EXCEPTION_DETAIL: int

COMPARISON_FLAGS: int

REPORT_UDIFF: int
REPORT_CDIFF: int
REPORT_NDIFF: int
REPORT_ONLY_FIRST_FAILURE: int
if sys.version_info >= (3, 4):
    FAIL_FAST: int

REPORTING_FLAGS: int

BLANKLINE_MARKER: str
ELLIPSIS_MARKER: str

class Example:
    source: str
    want: str
    exc_msg: Optional[str]
    lineno: int
    indent: int
    options: Dict[int, bool]
    def __init__(
        self,
        source: str,
        want: str,
        exc_msg: Optional[str] = ...,
        lineno: int = ...,
        indent: int = ...,
        options: Optional[Dict[int, bool]] = ...,
    ) -> None: ...
    def __hash__(self) -> int: ...

class DocTest:
    examples: List[Example]
    globs: Dict[str, Any]
    name: str
    filename: Optional[str]
    lineno: Optional[int]
    docstring: Optional[str]
    def __init__(
        self,
        examples: List[Example],
        globs: Dict[str, Any],
        name: str,
        filename: Optional[str],
        lineno: Optional[int],
        docstring: Optional[str],
    ) -> None: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: DocTest) -> bool: ...

class DocTestParser:
    def parse(self, string: str, name: str = ...) -> List[Union[str, Example]]: ...
    def get_doctest(
        self, string: str, globs: Dict[str, Any], name: str, filename: Optional[str], lineno: Optional[int]
    ) -> DocTest: ...
    def get_examples(self, string: str, name: str = ...) -> List[Example]: ...

class DocTestFinder:
    def __init__(
        self, verbose: bool = ..., parser: DocTestParser = ..., recurse: bool = ..., exclude_empty: bool = ...
    ) -> None: ...
    def find(
        self,
        obj: object,
        name: Optional[str] = ...,
        module: Union[None, bool, types.ModuleType] = ...,
        globs: Optional[Dict[str, Any]] = ...,
        extraglobs: Optional[Dict[str, Any]] = ...,
    ) -> List[DocTest]: ...

_Out = Callable[[str], Any]
_ExcInfo = Tuple[Type[BaseException], BaseException, types.TracebackType]

class DocTestRunner:
    DIVIDER: str
    optionflags: int
    original_optionflags: int
    tries: int
    failures: int
    test: DocTest
    def __init__(self, checker: Optional[OutputChecker] = ..., verbose: Optional[bool] = ..., optionflags: int = ...) -> None: ...
    def report_start(self, out: _Out, test: DocTest, example: Example) -> None: ...
    def report_success(self, out: _Out, test: DocTest, example: Example, got: str) -> None: ...
    def report_failure(self, out: _Out, test: DocTest, example: Example, got: str) -> None: ...
    def report_unexpected_exception(self, out: _Out, test: DocTest, example: Example, exc_info: _ExcInfo) -> None: ...
    def run(
        self, test: DocTest, compileflags: Optional[int] = ..., out: Optional[_Out] = ..., clear_globs: bool = ...
    ) -> TestResults: ...
    def summarize(self, verbose: Optional[bool] = ...) -> TestResults: ...
    def merge(self, other: DocTestRunner) -> None: ...

class OutputChecker:
    def check_output(self, want: str, got: str, optionflags: int) -> bool: ...
    def output_difference(self, example: Example, got: str, optionflags: int) -> str: ...

class DocTestFailure(Exception):
    test: DocTest
    example: Example
    got: str
    def __init__(self, test: DocTest, example: Example, got: str) -> None: ...

class UnexpectedException(Exception):
    test: DocTest
    example: Example
    exc_info: _ExcInfo
    def __init__(self, test: DocTest, example: Example, exc_info: _ExcInfo) -> None: ...

class DebugRunner(DocTestRunner): ...

master: Optional[DocTestRunner]

def testmod(
    m: Optional[types.ModuleType] = ...,
    name: Optional[str] = ...,
    globs: Optional[Dict[str, Any]] = ...,
    verbose: Optional[bool] = ...,
    report: bool = ...,
    optionflags: int = ...,
    extraglobs: Optional[Dict[str, Any]] = ...,
    raise_on_error: bool = ...,
    exclude_empty: bool = ...,
) -> TestResults: ...
def testfile(
    filename: str,
    module_relative: bool = ...,
    name: Optional[str] = ...,
    package: Union[None, str, types.ModuleType] = ...,
    globs: Optional[Dict[str, Any]] = ...,
    verbose: Optional[bool] = ...,
    report: bool = ...,
    optionflags: int = ...,
    extraglobs: Optional[Dict[str, Any]] = ...,
    raise_on_error: bool = ...,
    parser: DocTestParser = ...,
    encoding: Optional[str] = ...,
) -> TestResults: ...
def run_docstring_examples(
    f: object,
    globs: Dict[str, Any],
    verbose: bool = ...,
    name: str = ...,
    compileflags: Optional[int] = ...,
    optionflags: int = ...,
) -> None: ...
def set_unittest_reportflags(flags: int) -> int: ...

class DocTestCase(unittest.TestCase):
    def __init__(
        self,
        test: DocTest,
        optionflags: int = ...,
        setUp: Optional[Callable[[DocTest], Any]] = ...,
        tearDown: Optional[Callable[[DocTest], Any]] = ...,
        checker: Optional[OutputChecker] = ...,
    ) -> None: ...
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...
    def runTest(self) -> None: ...
    def format_failure(self, err: str) -> str: ...
    def debug(self) -> None: ...
    def id(self) -> str: ...
    def __hash__(self) -> int: ...
    def shortDescription(self) -> str: ...

class SkipDocTestCase(DocTestCase):
    def __init__(self, module: types.ModuleType) -> None: ...
    def setUp(self) -> None: ...
    def test_skip(self) -> None: ...
    def shortDescription(self) -> str: ...

if sys.version_info >= (3, 4):
    class _DocTestSuite(unittest.TestSuite): ...

else:
    _DocTestSuite = unittest.TestSuite

def DocTestSuite(
    module: Union[None, str, types.ModuleType] = ...,
    globs: Optional[Dict[str, Any]] = ...,
    extraglobs: Optional[Dict[str, Any]] = ...,
    test_finder: Optional[DocTestFinder] = ...,
    **options: Any,
) -> _DocTestSuite: ...

class DocFileCase(DocTestCase):
    def id(self) -> str: ...
    def format_failure(self, err: str) -> str: ...

def DocFileTest(
    path: str,
    module_relative: bool = ...,
    package: Union[None, str, types.ModuleType] = ...,
    globs: Optional[Dict[str, Any]] = ...,
    parser: DocTestParser = ...,
    encoding: Optional[str] = ...,
    **options: Any,
) -> DocFileCase: ...
def DocFileSuite(*paths: str, **kw: Any) -> _DocTestSuite: ...
def script_from_examples(s: str) -> str: ...
def testsource(module: Union[None, str, types.ModuleType], name: str) -> str: ...
def debug_src(src: str, pm: bool = ..., globs: Optional[Dict[str, Any]] = ...) -> None: ...
def debug_script(src: str, pm: bool = ..., globs: Optional[Dict[str, Any]] = ...) -> None: ...
def debug(module: Union[None, str, types.ModuleType], name: str, pm: bool = ...) -> None: ...



from typing import List, Optional

from django.db.models import Model

from nwon_django_toolbox.typings import AllowedUpdate


def allow_updates_for_model(
    model_instance: Model, disallowed_field_names: Optional[List[str]] = None
) -> List[AllowedUpdate]:
    """
    Returns an AllowedUpdate array which allows updating all fields except
    for the ones given by argument
    """

    fields = model_instance._meta.get_fields()
    return [
        AllowedUpdate(field_name=field.name,)
        for field in fields
        if disallowed_field_names == None or field.name not in disallowed_field_names
    ]



import enum
import sys
from typing import Any

if sys.version_info >= (3, 11):
    enum_property = enum.property
else:
    enum_property = property

class ChoicesMeta(enum.EnumMeta):
    names: list[str]
    choices: list[tuple[Any, str]]
    labels: list[str]
    values: list[Any]
    def __contains__(self, member: Any) -> bool: ...

class Choices(enum.Enum, metaclass=ChoicesMeta):
    @property
    def label(self) -> str: ...
    @enum_property
    def value(self) -> Any: ...

# fake
class _IntegerChoicesMeta(ChoicesMeta):
    names: list[str]
    choices: list[tuple[int, str]]
    labels: list[str]
    values: list[int]

class IntegerChoices(int, Choices, metaclass=_IntegerChoicesMeta):
    @enum_property
    def value(self) -> int: ...

# fake
class _TextChoicesMeta(ChoicesMeta):
    names: list[str]
    choices: list[tuple[str, str]]
    labels: list[str]
    values: list[str]

class TextChoices(str, Choices, metaclass=_TextChoicesMeta):
    @enum_property
    def value(self) -> str: ...



from typing_extensions import Literal

# These are not actually bools. See #4669
NO: bool
YES: bool
TRUE: bool
FALSE: bool
ON: bool
OFF: bool
N: Literal["n"]
S: Literal["s"]
W: Literal["w"]
E: Literal["e"]
NW: Literal["nw"]
SW: Literal["sw"]
NE: Literal["ne"]
SE: Literal["se"]
NS: Literal["ns"]
EW: Literal["ew"]
NSEW: Literal["nsew"]
CENTER: Literal["center"]
NONE: Literal["none"]
X: Literal["x"]
Y: Literal["y"]
BOTH: Literal["both"]
LEFT: Literal["left"]
TOP: Literal["top"]
RIGHT: Literal["right"]
BOTTOM: Literal["bottom"]
RAISED: Literal["raised"]
SUNKEN: Literal["sunken"]
FLAT: Literal["flat"]
RIDGE: Literal["ridge"]
GROOVE: Literal["groove"]
SOLID: Literal["solid"]
HORIZONTAL: Literal["horizontal"]
VERTICAL: Literal["vertical"]
NUMERIC: Literal["numeric"]
CHAR: Literal["char"]
WORD: Literal["word"]
BASELINE: Literal["baseline"]
INSIDE: Literal["inside"]
OUTSIDE: Literal["outside"]
SEL: Literal["sel"]
SEL_FIRST: Literal["sel.first"]
SEL_LAST: Literal["sel.last"]
END: Literal["end"]
INSERT: Literal["insert"]
CURRENT: Literal["current"]
ANCHOR: Literal["anchor"]
ALL: Literal["all"]
NORMAL: Literal["normal"]
DISABLED: Literal["disabled"]
ACTIVE: Literal["active"]
HIDDEN: Literal["hidden"]
CASCADE: Literal["cascade"]
CHECKBUTTON: Literal["checkbutton"]
COMMAND: Literal["command"]
RADIOBUTTON: Literal["radiobutton"]
SEPARATOR: Literal["separator"]
SINGLE: Literal["single"]
BROWSE: Literal["browse"]
MULTIPLE: Literal["multiple"]
EXTENDED: Literal["extended"]
DOTBOX: Literal["dotbox"]
UNDERLINE: Literal["underline"]
PIESLICE: Literal["pieslice"]
CHORD: Literal["chord"]
ARC: Literal["arc"]
FIRST: Literal["first"]
LAST: Literal["last"]
BUTT: Literal["butt"]
PROJECTING: Literal["projecting"]
ROUND: Literal["round"]
BEVEL: Literal["bevel"]
MITER: Literal["miter"]
MOVETO: Literal["moveto"]
SCROLL: Literal["scroll"]
UNITS: Literal["units"]
PAGES: Literal["pages"]



import datetime
from collections.abc import Collection, Iterator, Sequence
from decimal import Decimal
from re import Pattern
from typing import Any, Protocol
from uuid import UUID

from django.core.files import File
from django.core.validators import _ValidatorCallable
from django.db.models.fields import _Choice, _ChoiceNamedGroup, _ChoicesCallable, _ErrorMessagesT, _FieldChoices
from django.forms.boundfield import BoundField
from django.forms.forms import BaseForm
from django.forms.widgets import ChoiceWidget, Widget
from django.utils.datastructures import _PropertyDescriptor
from django.utils.functional import _StrOrPromise
from typing_extensions import TypeAlias

# Problem: attribute `widget` is always of type `Widget` after field instantiation.
# However, on class level it can be set to `Type[Widget]` too.
# If we annotate it as `Union[Widget, Type[Widget]]`, every code that uses field
# instances will not typecheck.
# If we annotate it as `Widget`, any widget subclasses that do e.g.
# `widget = Select` will not typecheck.
# `Any` gives too much freedom, but does not create false positives.
_ClassLevelWidgetT: TypeAlias = Any

class Field:
    initial: Any
    label: _StrOrPromise | None
    required: bool
    widget: _ClassLevelWidgetT
    hidden_widget: type[Widget]
    default_validators: list[_ValidatorCallable]
    default_error_messages: _ErrorMessagesT
    empty_values: Sequence[Any]
    show_hidden_initial: bool
    help_text: _StrOrPromise
    disabled: bool
    label_suffix: str | None
    localize: bool
    error_messages: _ErrorMessagesT
    validators: list[_ValidatorCallable]
    max_length: int | None
    def __init__(
        self,
        *,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def prepare_value(self, value: Any) -> Any: ...
    def to_python(self, value: Any | None) -> Any | None: ...
    def validate(self, value: Any) -> None: ...
    def run_validators(self, value: Any) -> None: ...
    def clean(self, value: Any) -> Any: ...
    def bound_data(self, data: Any, initial: Any) -> Any: ...
    def widget_attrs(self, widget: Widget) -> dict[str, Any]: ...
    def has_changed(self, initial: Any | None, data: Any | None) -> bool: ...
    def get_bound_field(self, form: BaseForm, field_name: str) -> BoundField: ...
    def deconstruct(self) -> Any: ...

class CharField(Field):
    max_length: int | None
    min_length: int | None
    strip: bool
    empty_value: str | None
    def __init__(
        self,
        *,
        max_length: int | None = ...,
        min_length: int | None = ...,
        strip: bool = ...,
        empty_value: str | None = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def to_python(self, value: Any | None) -> str | None: ...
    def widget_attrs(self, widget: Widget) -> dict[str, Any]: ...

class IntegerField(Field):
    max_value: int | None
    min_value: int | None
    re_decimal: Any
    def __init__(
        self,
        *,
        max_value: int | None = ...,
        min_value: int | None = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def to_python(self, value: Any | None) -> int | None: ...
    def widget_attrs(self, widget: Widget) -> dict[str, Any]: ...

class FloatField(IntegerField):
    def __init__(
        self,
        *,
        max_value: int | float | None = ...,
        min_value: int | float | None = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def to_python(self, value: Any | None) -> float | None: ...  # type: ignore
    def validate(self, value: float) -> None: ...
    def widget_attrs(self, widget: Widget) -> dict[str, Any]: ...

class DecimalField(IntegerField):
    decimal_places: int | None
    max_digits: int | None
    def __init__(
        self,
        *,
        max_value: Decimal | int | float | None = ...,
        min_value: Decimal | int | float | None = ...,
        max_digits: int | None = ...,
        decimal_places: int | None = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def to_python(self, value: Any | None) -> Decimal | None: ...  # type: ignore
    def validate(self, value: Decimal) -> None: ...
    def widget_attrs(self, widget: Widget) -> dict[str, Any]: ...

class BaseTemporalField(Field):
    input_formats: Any
    def __init__(
        self,
        *,
        input_formats: Any | None = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def to_python(self, value: str | None) -> Any | None: ...
    def strptime(self, value: str, format: str) -> Any: ...

class DateField(BaseTemporalField):
    def to_python(self, value: None | str | datetime.datetime | datetime.date) -> datetime.date | None: ...
    def strptime(self, value: str, format: str) -> datetime.date: ...

class TimeField(BaseTemporalField):
    def to_python(self, value: None | str | datetime.time) -> datetime.time | None: ...
    def strptime(self, value: str, format: str) -> datetime.time: ...

class DateTimeFormatsIterator:
    def __iter__(self) -> Iterator[str]: ...

class DateTimeField(BaseTemporalField):
    def to_python(self, value: None | str | datetime.datetime | datetime.date) -> datetime.datetime | None: ...
    def strptime(self, value: str, format: str) -> datetime.datetime: ...

class DurationField(Field):
    def prepare_value(self, value: datetime.timedelta | str | None) -> str | None: ...
    def to_python(self, value: Any | None) -> datetime.timedelta | None: ...

class RegexField(CharField):
    regex: _PropertyDescriptor[str | Pattern[str], Pattern[str]]
    def __init__(
        self,
        regex: str | Pattern[str],
        *,
        max_length: int | None = ...,
        min_length: int | None = ...,
        strip: bool = ...,
        empty_value: str | None = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...

class EmailField(CharField):
    def __init__(
        self,
        *,
        max_length: int | None = ...,
        min_length: int | None = ...,
        strip: bool = ...,
        empty_value: str | None = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...

class FileField(Field):
    allow_empty_file: bool
    def __init__(
        self,
        *,
        max_length: int | None = ...,
        allow_empty_file: bool = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def clean(self, data: Any, initial: Any | None = ...) -> Any: ...
    def to_python(self, data: File | None) -> File | None: ...
    def bound_data(self, data: Any | None, initial: Any) -> Any: ...
    def has_changed(self, initial: Any | None, data: Any | None) -> bool: ...

class ImageField(FileField):
    def to_python(self, data: File | None) -> File | None: ...
    def widget_attrs(self, widget: Widget) -> dict[str, Any]: ...

class URLField(CharField):
    def __init__(
        self,
        *,
        max_length: int | None = ...,
        min_length: int | None = ...,
        strip: bool = ...,
        empty_value: str | None = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def to_python(self, value: Any | None) -> str | None: ...

class BooleanField(Field):
    def to_python(self, value: Any | None) -> bool: ...
    def validate(self, value: Any) -> None: ...
    def has_changed(self, initial: Any | None, data: Any | None) -> bool: ...

class NullBooleanField(BooleanField):
    def to_python(self, value: Any | None) -> bool | None: ...  # type: ignore
    def validate(self, value: Any) -> None: ...

class CallableChoiceIterator:
    choices_func: _ChoicesCallable
    def __init__(self, choices_func: _ChoicesCallable) -> None: ...
    def __iter__(self) -> Iterator[_Choice | _ChoiceNamedGroup]: ...

class ChoiceField(Field):
    choices: _PropertyDescriptor[
        _FieldChoices | _ChoicesCallable | CallableChoiceIterator,
        _FieldChoices | CallableChoiceIterator,
    ]
    widget: _ClassLevelWidgetT
    def __init__(
        self,
        *,
        choices: _FieldChoices | _ChoicesCallable = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    # Real return type of `to_python` is `str`, but it results in errors when
    # subclassing `ModelChoiceField`: `# type: ignore[override]` is not inherited
    def to_python(self, value: Any | None) -> Any: ...
    def validate(self, value: Any) -> None: ...
    def valid_value(self, value: Any) -> bool: ...

class _CoerceCallable(Protocol):
    def __call__(self, __value: Any) -> Any: ...

class TypedChoiceField(ChoiceField):
    coerce: _CoerceCallable
    empty_value: str | None
    def __init__(
        self,
        *,
        coerce: _CoerceCallable = ...,
        empty_value: str | None = ...,
        choices: _FieldChoices | _ChoicesCallable = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def clean(self, value: Any) -> Any: ...

class MultipleChoiceField(ChoiceField):
    def to_python(self, value: Any | None) -> list[str]: ...
    def validate(self, value: Any) -> None: ...
    def has_changed(self, initial: Collection[Any] | None, data: Collection[Any] | None) -> bool: ...

class TypedMultipleChoiceField(MultipleChoiceField):
    coerce: _CoerceCallable
    empty_value: list[Any] | None
    def __init__(
        self,
        *,
        coerce: _CoerceCallable = ...,
        empty_value: list[Any] | None = ...,
        choices: _FieldChoices | _ChoicesCallable = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def clean(self, value: Any) -> Any: ...
    def validate(self, value: Any) -> None: ...

class ComboField(Field):
    fields: Sequence[Field]
    def __init__(
        self,
        fields: Sequence[Field],
        *,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def clean(self, value: Any) -> Any: ...

class MultiValueField(Field):
    require_all_fields: bool
    fields: Sequence[Field]
    def __init__(
        self,
        fields: Sequence[Field],
        *,
        require_all_fields: bool = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def compress(self, data_list: Any) -> Any: ...
    def has_changed(self, initial: Any | None, data: Any | None) -> bool: ...
    def clean(self, value: Any) -> Any: ...
    def validate(self, value: Any) -> None: ...

class FilePathField(ChoiceField):
    allow_files: bool
    allow_folders: bool
    match: str | None
    path: str
    recursive: bool
    match_re: Pattern[str] | None
    def __init__(
        self,
        path: str,
        *,
        match: str | None = ...,
        recursive: bool = ...,
        allow_files: bool = ...,
        allow_folders: bool = ...,
        choices: _FieldChoices | _ChoicesCallable = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...

class SplitDateTimeField(MultiValueField):
    def __init__(
        self,
        *,
        input_date_formats: Any | None = ...,
        input_time_formats: Any | None = ...,
        fields: Sequence[Field] = ...,
        require_all_fields: bool = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def compress(self, data_list: tuple[datetime.date, datetime.time] | None) -> datetime.datetime | None: ...

class GenericIPAddressField(CharField):
    unpack_ipv4: bool
    def __init__(
        self,
        *,
        protocol: str = ...,
        unpack_ipv4: bool = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...
    def to_python(self, value: Any) -> str: ...

class SlugField(CharField):
    allow_unicode: bool
    def __init__(
        self,
        *,
        allow_unicode: bool = ...,
        max_length: Any | None = ...,
        min_length: Any | None = ...,
        strip: bool = ...,
        empty_value: str | None = ...,
        required: bool = ...,
        widget: Widget | type[Widget] | None = ...,
        label: _StrOrPromise | None = ...,
        initial: Any | None = ...,
        help_text: _StrOrPromise = ...,
        error_messages: _ErrorMessagesT | None = ...,
        show_hidden_initial: bool = ...,
        validators: Sequence[_ValidatorCallable] = ...,
        localize: bool = ...,
        disabled: bool = ...,
        label_suffix: str | None = ...,
    ) -> None: ...

class UUIDField(CharField):
    def prepare_value(self, value: Any | None) -> Any | None: ...
    def to_python(self, value: Any) -> UUID | None: ...  # type: ignore

class InvalidJSONInput(str): ...
class JSONString(str): ...

class JSONField(CharField):
    default_error_messages: _ErrorMessagesT
    widget: _ClassLevelWidgetT
    encoder: Any
    decoder: Any
    def __init__(self, encoder: Any | None = ..., decoder: Any | None = ..., **kwargs: Any) -> None: ...
    def to_python(self, value: Any) -> Any: ...
    def bound_data(self, data: Any, initial: Any) -> Any: ...
    def prepare_value(self, value: Any) -> str: ...
    def has_changed(self, initial: Any | None, data: Any | None) -> bool: ...



import os
from pathlib import Path

from typing_extensions import TypeAlias

_PathCompatible: TypeAlias = str | os.PathLike[str]

def safe_join(base: _PathCompatible, *paths: _PathCompatible) -> str: ...
def symlinks_supported() -> bool: ...
def to_path(value: Path | str) -> Path: ...



import sys
from _typeshed import SupportsRichComparisonT
from collections.abc import Hashable, Iterable, Sequence
from decimal import Decimal
from fractions import Fraction
from typing import Any, NamedTuple, SupportsFloat, TypeVar
from typing_extensions import Literal, Self, TypeAlias

__all__ = [
    "StatisticsError",
    "pstdev",
    "pvariance",
    "stdev",
    "variance",
    "median",
    "median_low",
    "median_high",
    "median_grouped",
    "mean",
    "mode",
    "harmonic_mean",
]

if sys.version_info >= (3, 8):
    __all__ += ["geometric_mean", "multimode", "NormalDist", "fmean", "quantiles"]

if sys.version_info >= (3, 10):
    __all__ += ["covariance", "correlation", "linear_regression"]

# Most functions in this module accept homogeneous collections of one of these types
_Number: TypeAlias = float | Decimal | Fraction
_NumberT = TypeVar("_NumberT", float, Decimal, Fraction)

# Used in mode, multimode
_HashableT = TypeVar("_HashableT", bound=Hashable)

class StatisticsError(ValueError): ...

if sys.version_info >= (3, 11):
    def fmean(data: Iterable[SupportsFloat], weights: Iterable[SupportsFloat] | None = None) -> float: ...

elif sys.version_info >= (3, 8):
    def fmean(data: Iterable[SupportsFloat]) -> float: ...

if sys.version_info >= (3, 8):
    def geometric_mean(data: Iterable[SupportsFloat]) -> float: ...

def mean(data: Iterable[_NumberT]) -> _NumberT: ...

if sys.version_info >= (3, 10):
    def harmonic_mean(data: Iterable[_NumberT], weights: Iterable[_Number] | None = None) -> _NumberT: ...

else:
    def harmonic_mean(data: Iterable[_NumberT]) -> _NumberT: ...

def median(data: Iterable[_NumberT]) -> _NumberT: ...
def median_low(data: Iterable[SupportsRichComparisonT]) -> SupportsRichComparisonT: ...
def median_high(data: Iterable[SupportsRichComparisonT]) -> SupportsRichComparisonT: ...

if sys.version_info >= (3, 11):
    def median_grouped(data: Iterable[SupportsFloat], interval: SupportsFloat = 1.0) -> float: ...

else:
    def median_grouped(data: Iterable[_NumberT], interval: _NumberT | float = 1) -> _NumberT | float: ...

def mode(data: Iterable[_HashableT]) -> _HashableT: ...

if sys.version_info >= (3, 8):
    def multimode(data: Iterable[_HashableT]) -> list[_HashableT]: ...

def pstdev(data: Iterable[_NumberT], mu: _NumberT | None = None) -> _NumberT: ...
def pvariance(data: Iterable[_NumberT], mu: _NumberT | None = None) -> _NumberT: ...

if sys.version_info >= (3, 8):
    def quantiles(
        data: Iterable[_NumberT], *, n: int = 4, method: Literal["inclusive", "exclusive"] = "exclusive"
    ) -> list[_NumberT]: ...

def stdev(data: Iterable[_NumberT], xbar: _NumberT | None = None) -> _NumberT: ...
def variance(data: Iterable[_NumberT], xbar: _NumberT | None = None) -> _NumberT: ...

if sys.version_info >= (3, 8):
    class NormalDist:
        def __init__(self, mu: float = 0.0, sigma: float = 1.0) -> None: ...
        @property
        def mean(self) -> float: ...
        @property
        def median(self) -> float: ...
        @property
        def mode(self) -> float: ...
        @property
        def stdev(self) -> float: ...
        @property
        def variance(self) -> float: ...
        @classmethod
        def from_samples(cls, data: Iterable[SupportsFloat]) -> Self: ...
        def samples(self, n: int, *, seed: Any | None = None) -> list[float]: ...
        def pdf(self, x: float) -> float: ...
        def cdf(self, x: float) -> float: ...
        def inv_cdf(self, p: float) -> float: ...
        def overlap(self, other: NormalDist) -> float: ...
        def quantiles(self, n: int = 4) -> list[float]: ...
        if sys.version_info >= (3, 9):
            def zscore(self, x: float) -> float: ...

        def __eq__(self, x2: object) -> bool: ...
        def __add__(self, x2: float | NormalDist) -> NormalDist: ...
        def __sub__(self, x2: float | NormalDist) -> NormalDist: ...
        def __mul__(self, x2: float) -> NormalDist: ...
        def __truediv__(self, x2: float) -> NormalDist: ...
        def __pos__(self) -> NormalDist: ...
        def __neg__(self) -> NormalDist: ...
        __radd__ = __add__
        def __rsub__(self, x2: float | NormalDist) -> NormalDist: ...
        __rmul__ = __mul__

if sys.version_info >= (3, 10):
    def correlation(__x: Sequence[_Number], __y: Sequence[_Number]) -> float: ...
    def covariance(__x: Sequence[_Number], __y: Sequence[_Number]) -> float: ...

    class LinearRegression(NamedTuple):
        slope: float
        intercept: float

if sys.version_info >= (3, 11):
    def linear_regression(
        __regressor: Sequence[_Number], __dependent_variable: Sequence[_Number], *, proportional: bool = False
    ) -> LinearRegression: ...

elif sys.version_info >= (3, 10):
    def linear_regression(__regressor: Sequence[_Number], __dependent_variable: Sequence[_Number]) -> LinearRegression: ...



from typing import Any

from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import Serializer
from rest_framework.views import APIView, AsView, GenericView

class ObtainAuthToken(APIView):
    serializer_class: type[Serializer]
    def post(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...

obtain_auth_token: AsView[GenericView]



import io
from _typeshed import ReadableBuffer, StrOrBytesPath
from collections.abc import Mapping, Sequence
from typing import IO, Any, TextIO, overload
from typing_extensions import Literal, Self, TypeAlias, final

__all__ = [
    "CHECK_NONE",
    "CHECK_CRC32",
    "CHECK_CRC64",
    "CHECK_SHA256",
    "CHECK_ID_MAX",
    "CHECK_UNKNOWN",
    "FILTER_LZMA1",
    "FILTER_LZMA2",
    "FILTER_DELTA",
    "FILTER_X86",
    "FILTER_IA64",
    "FILTER_ARM",
    "FILTER_ARMTHUMB",
    "FILTER_POWERPC",
    "FILTER_SPARC",
    "FORMAT_AUTO",
    "FORMAT_XZ",
    "FORMAT_ALONE",
    "FORMAT_RAW",
    "MF_HC3",
    "MF_HC4",
    "MF_BT2",
    "MF_BT3",
    "MF_BT4",
    "MODE_FAST",
    "MODE_NORMAL",
    "PRESET_DEFAULT",
    "PRESET_EXTREME",
    "LZMACompressor",
    "LZMADecompressor",
    "LZMAFile",
    "LZMAError",
    "open",
    "compress",
    "decompress",
    "is_check_supported",
]

_OpenBinaryWritingMode: TypeAlias = Literal["w", "wb", "x", "xb", "a", "ab"]
_OpenTextWritingMode: TypeAlias = Literal["wt", "xt", "at"]

_PathOrFile: TypeAlias = StrOrBytesPath | IO[bytes]

_FilterChain: TypeAlias = Sequence[Mapping[str, Any]]

FORMAT_AUTO: Literal[0]
FORMAT_XZ: Literal[1]
FORMAT_ALONE: Literal[2]
FORMAT_RAW: Literal[3]
CHECK_NONE: Literal[0]
CHECK_CRC32: Literal[1]
CHECK_CRC64: Literal[4]
CHECK_SHA256: Literal[10]
CHECK_ID_MAX: Literal[15]
CHECK_UNKNOWN: Literal[16]
FILTER_LZMA1: int  # v big number
FILTER_LZMA2: Literal[33]
FILTER_DELTA: Literal[3]
FILTER_X86: Literal[4]
FILTER_IA64: Literal[6]
FILTER_ARM: Literal[7]
FILTER_ARMTHUMB: Literal[8]
FILTER_SPARC: Literal[9]
FILTER_POWERPC: Literal[5]
MF_HC3: Literal[3]
MF_HC4: Literal[4]
MF_BT2: Literal[18]
MF_BT3: Literal[19]
MF_BT4: Literal[20]
MODE_FAST: Literal[1]
MODE_NORMAL: Literal[2]
PRESET_DEFAULT: Literal[6]
PRESET_EXTREME: int  # v big number

# from _lzma.c
@final
class LZMADecompressor:
    def __init__(self, format: int | None = ..., memlimit: int | None = ..., filters: _FilterChain | None = ...) -> None: ...
    def decompress(self, data: ReadableBuffer, max_length: int = -1) -> bytes: ...
    @property
    def check(self) -> int: ...
    @property
    def eof(self) -> bool: ...
    @property
    def unused_data(self) -> bytes: ...
    @property
    def needs_input(self) -> bool: ...

# from _lzma.c
@final
class LZMACompressor:
    def __init__(
        self, format: int | None = ..., check: int = ..., preset: int | None = ..., filters: _FilterChain | None = ...
    ) -> None: ...
    def compress(self, __data: ReadableBuffer) -> bytes: ...
    def flush(self) -> bytes: ...

class LZMAError(Exception): ...

class LZMAFile(io.BufferedIOBase, IO[bytes]):  # type: ignore[misc]  # incompatible definitions of writelines in the base classes
    def __init__(
        self,
        filename: _PathOrFile | None = None,
        mode: str = "r",
        *,
        format: int | None = None,
        check: int = -1,
        preset: int | None = None,
        filters: _FilterChain | None = None,
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def peek(self, size: int = -1) -> bytes: ...
    def read(self, size: int | None = -1) -> bytes: ...
    def read1(self, size: int = -1) -> bytes: ...
    def readline(self, size: int | None = -1) -> bytes: ...
    def write(self, data: ReadableBuffer) -> int: ...
    def seek(self, offset: int, whence: int = 0) -> int: ...

@overload
def open(
    filename: _PathOrFile,
    mode: Literal["r", "rb"] = "rb",
    *,
    format: int | None = None,
    check: Literal[-1] = -1,
    preset: None = None,
    filters: _FilterChain | None = None,
    encoding: None = None,
    errors: None = None,
    newline: None = None,
) -> LZMAFile: ...
@overload
def open(
    filename: _PathOrFile,
    mode: _OpenBinaryWritingMode,
    *,
    format: int | None = None,
    check: int = -1,
    preset: int | None = None,
    filters: _FilterChain | None = None,
    encoding: None = None,
    errors: None = None,
    newline: None = None,
) -> LZMAFile: ...
@overload
def open(
    filename: StrOrBytesPath,
    mode: Literal["rt"],
    *,
    format: int | None = None,
    check: Literal[-1] = -1,
    preset: None = None,
    filters: _FilterChain | None = None,
    encoding: str | None = None,
    errors: str | None = None,
    newline: str | None = None,
) -> TextIO: ...
@overload
def open(
    filename: StrOrBytesPath,
    mode: _OpenTextWritingMode,
    *,
    format: int | None = None,
    check: int = -1,
    preset: int | None = None,
    filters: _FilterChain | None = None,
    encoding: str | None = None,
    errors: str | None = None,
    newline: str | None = None,
) -> TextIO: ...
@overload
def open(
    filename: _PathOrFile,
    mode: str,
    *,
    format: int | None = None,
    check: int = -1,
    preset: int | None = None,
    filters: _FilterChain | None = None,
    encoding: str | None = None,
    errors: str | None = None,
    newline: str | None = None,
) -> LZMAFile | TextIO: ...
def compress(
    data: ReadableBuffer, format: int = 1, check: int = -1, preset: int | None = None, filters: _FilterChain | None = None
) -> bytes: ...
def decompress(
    data: ReadableBuffer, format: int = 0, memlimit: int | None = None, filters: _FilterChain | None = None
) -> bytes: ...
def is_check_supported(__check_id: int) -> bool: ...



from collections.abc import Callable
from typing import Any

from django.contrib.gis.db.backends.base.operations import BaseSpatialOperations
from django.contrib.gis.db.backends.utils import SpatialOperator
from django.contrib.gis.geos.geometry import GEOSGeometryBase
from django.db.backends.mysql.operations import DatabaseOperations

class MySQLOperations(BaseSpatialOperations, DatabaseOperations):
    name: str
    geom_func_prefix: str
    Adapter: Any
    @property
    def mariadb(self) -> bool: ...
    @property
    def mysql(self) -> bool: ...  # type: ignore
    @property
    def select(self) -> str: ...  # type: ignore
    @property
    def from_text(self) -> str: ...  # type: ignore
    @property
    def gis_operators(self) -> dict[str, SpatialOperator]: ...
    disallowed_aggregates: Any
    @property
    def unsupported_functions(self) -> set[str]: ...  # type: ignore
    def geo_db_type(self, f: Any) -> Any: ...
    def get_distance(self, f: Any, value: Any, lookup_type: Any) -> list[Any]: ...
    def get_geometry_converter(self, expression: Any) -> Callable[[Any, Any, Any], GEOSGeometryBase | None]: ...



from collections.abc import Callable, Sequence
from types import ModuleType
from typing import Any, overload

from django.urls import URLPattern, URLResolver, _AnyURL
from typing_extensions import TypeAlias

from ..conf.urls import IncludedURLConf
from ..http.response import HttpResponseBase

_URLConf: TypeAlias = str | ModuleType | Sequence[_AnyURL]

def include(
    arg: _URLConf | tuple[_URLConf, str], namespace: str | None = ...
) -> tuple[Sequence[URLResolver | URLPattern], str | None, str | None]: ...

# path()
@overload
def path(
    route: str, view: Callable[..., HttpResponseBase], kwargs: dict[str, Any] = ..., name: str = ...
) -> URLPattern: ...
@overload
def path(route: str, view: IncludedURLConf, kwargs: dict[str, Any] = ..., name: str = ...) -> URLResolver: ...
@overload
def path(
    route: str, view: Sequence[URLResolver | str], kwargs: dict[str, Any] = ..., name: str = ...
) -> URLResolver: ...

# re_path()
@overload
def re_path(
    route: str, view: Callable[..., HttpResponseBase], kwargs: dict[str, Any] = ..., name: str = ...
) -> URLPattern: ...
@overload
def re_path(route: str, view: IncludedURLConf, kwargs: dict[str, Any] = ..., name: str = ...) -> URLResolver: ...
@overload
def re_path(
    route: str, view: Sequence[URLResolver | str], kwargs: dict[str, Any] = ..., name: str = ...
) -> URLResolver: ...



from types import TracebackType
from typing_extensions import Self, final

__all__ = ("Timeout", "timeout", "timeout_at")

@final
class Timeout:
    def __init__(self, when: float | None) -> None: ...
    def when(self) -> float | None: ...
    def reschedule(self, when: float | None) -> None: ...
    def expired(self) -> bool: ...
    async def __aenter__(self) -> Self: ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

def timeout(delay: float | None) -> Timeout: ...
def timeout_at(when: float | None) -> Timeout: ...



from typing import List, Optional, Type

from django.core.exceptions import ValidationError
from django.db.models import JSONField
from pydantic import BaseModel
from pydantic import ValidationError as PydanticValidationError


class PydanticJsonField(JSONField):
    def __init__(
        self, *args, pydantic_models: Optional[List[Type[BaseModel]]] = None, **kwargs
    ):
        self.pydantic_models: List[Type[BaseModel]] = (
            pydantic_models if pydantic_models else []
        )

        kwargs_to_pass = kwargs
        kwargs_to_pass.pop("pydantic_models", None)

        super().__init__(*args, **kwargs_to_pass)

    def _validate_schema(self, value):
        # Disable validation when migrations are faked
        if self.model.__module__ == "__fake__":
            return True

        validation_errors = []

        for model in self.pydantic_models:
            try:
                if isinstance(value, str):
                    model.parse_raw(value)
                else:
                    model.parse_obj(value)

                # Unset validation errors on successful parsing and break loop
                validation_errors = []
                break

            except PydanticValidationError as exc:
                validation_errors.append(
                    f"JSON does not fit Pydantic model {model.__name__} {format(exc)}"
                )

        if validation_errors.__len__() > 0:
            raise ValidationError(validation_errors)

        return value

    def validate(self, value, model_instance):
        super().validate(value, model_instance)
        self._validate_schema(value)

    def pre_save(self, model_instance, add):
        value = super().pre_save(model_instance, add)
        if value and not self.null:
            self._validate_schema(value)
        return value



from datetime import datetime
from typing import (
    Any,
    Callable,
    Dict,
    Generic,
    Iterable,
    List,
    Mapping,
    Optional,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import celery
import kombu
from celery.app.base import Celery
from celery.app.task import Task
from celery.result import EagerResult
from celery.utils import abstract

_F = TypeVar("_F", bound=Callable[..., Any])
_R = TypeVar("_R")

class Signature(Dict[str, Any], Generic[_R]):
    def __init__(
        self,
        task: Task[Any, _R] | str | None = ...,
        args: Optional[Tuple[Any, ...]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        options: Optional[Dict[str, Any]] = ...,
        type: Optional[Any] = ...,
        subtask_type: Optional[Any] = ...,
        immutable: bool = ...,
        app: Optional[Celery] = ...,
        *,
        # **ex expanded
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Signature[Any]] = ...,
        link_error: Optional[Signature[Any]] = ...,
        shadow: Optional[str] = ...,
        # apply_async options
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
    ) -> None: ...
    def __call__(self, *partial_args: Any, **partial_kwargs: Any) -> _R: ...
    def delay(
        self, *partial_args: Any, **partial_kwargs: Any
    ) -> celery.result.AsyncResult[_R]: ...
    def apply(
        self,
        args: Optional[Tuple[Any]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        **options: Any,
    ) -> EagerResult[_R]: ...
    def apply_async(
        self,
        args: Optional[Tuple[Any]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        route_name: Optional[str] = ...,
        *,
        # options
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Union[Signature[Any], List[Signature[Any]]]] = ...,
        link_error: Optional[Union[Signature[Any], List[Signature[Any]]]] = ...,
        shadow: Optional[str] = ...,
        # apply_async options
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
    ) -> celery.result.AsyncResult[_R]: ...
    def clone(
        self,
        args: Optional[Tuple[Any, ...]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        *,
        # **ex expanded
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Signature[Any]] = ...,
        link_error: Optional[Signature[Any]] = ...,
        shadow: Optional[str] = ...,
        # apply_async options
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
    ) -> Signature[_R]: ...
    partial = clone
    def freeze(
        self,
        _id: Optional[str] = ...,
        group_id: Optional[str] = ...,
        chord: Optional[chord] = ...,
        root_id: Optional[str] = ...,
        parent_id: Optional[str] = ...,
    ) -> celery.result.AsyncResult[_R]: ...
    def replace(
        self,
        args: Optional[Tuple[Any, ...]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        options: Optional[Dict[str, Any]] = ...,
    ) -> Signature[_R]: ...
    def set(
        self,
        immutable: Optional[bool] = ...,
        *,
        # **options expanded
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Signature[Any]] = ...,
        link_error: Optional[Signature[Any]] = ...,
        shadow: Optional[str] = ...,
        # apply_async options
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
    ) -> Signature[_R]: ...
    def set_immutable(self, immutable: bool) -> None: ...
    def append_to_list_option(self, key: str, value: Any) -> Any: ...
    def extend_list_option(self, key: str, value: Any) -> None: ...
    def link(self, callback: _F) -> _F: ...
    def link_error(self, errback: Callable[..., Any]) -> Signature[_R]: ...
    def on_error(self, errback: _F) -> _F: ...
    def flatten_links(self) -> List[Signature[Any]]: ...
    # TODO(sbdchd): use overloads to properly type this
    def __or__(self, other: Signature[Any]) -> Signature[Any]: ...
    def election(self) -> celery.result.AsyncResult[_R]: ...
    @property
    def name(self) -> str: ...
    @property
    def type(self) -> Any: ...
    @property
    def app(self) -> Celery: ...
    def AsyncResult(self) -> celery.result.AsyncResult[_R]: ...
    id: Optional[str]
    parent_id: Optional[str]
    root_id: Optional[str]
    task: Optional[str]
    args: Tuple[Any, ...]
    kwargs: Dict[str, Any]
    options: Dict[str, Any]
    subtask_type: Any
    chord_size: Optional[int]
    immutable: bool

class _chain(Signature[Any]):
    def __init__(
        self,
        *tasks: Signature[Any],
        # Signature extras
        options: Optional[Dict[str, Any]] = ...,
        type: Optional[Any] = ...,
        subtask_type: Optional[Any] = ...,
        immutable: bool = ...,
        app: Optional[Celery] = ...,
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Signature[Any]] = ...,
        link_error: Optional[Signature[Any]] = ...,
        shadow: Optional[str] = ...,
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
    ) -> None: ...

class chain(_chain): ...

class _basemap(Signature[Any]):
    def __init__(
        self,
        task: Optional[Task[Any, Any]],
        it: Iterable[Any],
        *,
        # Signature extras
        args: Optional[Tuple[Any, ...]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        options: Optional[Dict[str, Any]] = ...,
        type: Optional[Any] = ...,
        subtask_type: Optional[Any] = ...,
        immutable: bool = ...,
        app: Optional[Celery] = ...,
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Signature[Any]] = ...,
        link_error: Optional[Signature[Any]] = ...,
        shadow: Optional[str] = ...,
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
    ) -> None: ...

class xmap(_basemap): ...
class xstarmap(_basemap): ...

class chunks(Signature[Any]):
    def __init__(
        self,
        task: Optional[Task[Any, Any]],
        it: Iterable[Any],
        n: int,
        *,
        # Signature extras
        args: Optional[Tuple[Any, ...]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        options: Optional[Dict[str, Any]] = ...,
        type: Optional[Any] = ...,
        subtask_type: Optional[Any] = ...,
        immutable: bool = ...,
        app: Optional[Celery] = ...,
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Signature[Any]] = ...,
        link_error: Optional[Signature[Any]] = ...,
        shadow: Optional[str] = ...,
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
    ) -> None: ...
    def group(self) -> _group: ...

class group(Signature[Any]):
    @overload
    def __init__(
        self, __tasks: group | abstract.CallableSignature | Iterable[Signature[Any]]
    ) -> None: ...
    @overload
    def __init__(
        self,
        *tasks: Signature[Any],
        # Signature extras
        args: Optional[Tuple[Any, ...]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        options: Optional[Dict[str, Any]] = ...,
        type: Optional[Any] = ...,
        subtask_type: Optional[Any] = ...,
        immutable: bool = ...,
        app: Optional[Celery] = ...,
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Signature[Any]] = ...,
        link_error: Optional[Signature[Any]] = ...,
        shadow: Optional[str] = ...,
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
    ) -> None: ...
    def skew(
        self, start: float = ..., stop: float | None = ..., step: float = ...
    ) -> group: ...
    def __or__(self, other: Signature[Any]) -> chord: ...

_group = group

class chord(Signature[Any]):
    def __init__(
        self,
        header: Any,
        body: Optional[Any] = ...,
        task: str = ...,
        args: Optional[Tuple[Any, ...]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        app: Optional[Celery] = ...,
        *,
        # from Signature
        options: Optional[Dict[str, Any]] = ...,
        type: Optional[Any] = ...,
        subtask_type: Optional[Any] = ...,
        immutable: bool = ...,
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Signature[Any]] = ...,
        link_error: Optional[Signature[Any]] = ...,
        shadow: Optional[str] = ...,
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
    ) -> None: ...
    def __or__(self, other: Signature[Any]) -> chord: ...
    def __call__(  # type: ignore[override]
        self,
        body: Optional[Signature[Any]] = ...,
        **options: Any,
    ) -> celery.result.AsyncResult[Any]: ...

def signature(
    varies: Signature[Any] | str | Dict[str, Any], *args: Any, **kwargs: Any
) -> Signature[Any]: ...

subtask = signature

def maybe_signature(
    d: Optional[Union[abstract.CallableSignature, Mapping[str, Any]]],
    app: Optional[Celery] = ...,
    clone: bool = ...,
) -> Optional[abstract.CallableSignature]: ...

maybe_subtask = maybe_signature



from nwon_django_toolbox.exception_handler.api_exception_handler import (
    api_exception_handler,
)
from nwon_django_toolbox.exception_handler.exception_middleware import (
    ExceptionMiddleware,
)

__all__ = ["api_exception_handler", "ExceptionMiddleware"]



from collections.abc import Callable
from typing import Any, Protocol

from django.http.request import HttpRequest
from django.http.response import HttpResponse
from typing_extensions import TypeAlias

class RemovedInDjango40Warning(DeprecationWarning): ...
class RemovedInDjango41Warning(PendingDeprecationWarning): ...

RemovedInNextVersionWarning: TypeAlias = RemovedInDjango40Warning

class warn_about_renamed_method:
    class_name: str
    old_method_name: str
    new_method_name: str
    deprecation_warning: type[DeprecationWarning]
    def __init__(
        self, class_name: str, old_method_name: str, new_method_name: str, deprecation_warning: type[DeprecationWarning]
    ) -> None: ...
    def __call__(self, f: Callable) -> Callable: ...

class RenameMethodsBase(type):
    renamed_methods: Any
    def __new__(cls, name: Any, bases: Any, attrs: Any) -> type: ...

class DeprecationInstanceCheck(type):
    alternative: str
    deprecation_warning: type[Warning]
    def __instancecheck__(self, instance: Any) -> bool: ...

class GetResponseCallable(Protocol):
    def __call__(self, __request: HttpRequest) -> HttpResponse: ...

class MiddlewareMixin:
    get_response: GetResponseCallable
    def __init__(self, get_response: GetResponseCallable = ...) -> None: ...
    def __call__(self, request: HttpRequest) -> HttpResponse: ...



from typing import Optional

from nwon_baseline.typings import AnyDict
from rest_framework.response import Response
from rest_framework.test import APIClient as RestApiClient

from nwon_django_toolbox.settings import NWON_DJANGO_SETTINGS
from nwon_django_toolbox.typings import RequestBodyFormat


class ApiClient:
    """
    Facilitating request against URLs.

    Easy to set authentication token on initialization.
    """

    api_client: RestApiClient

    def __init__(
        self,
        token: Optional[str],
        authorization_prefix: str = NWON_DJANGO_SETTINGS.authorization_prefix,
    ):
        self.api_client = RestApiClient()

        if token:
            self.set_bearer_token(token, authorization_prefix)

    def set_bearer_token(
        self,
        token: str,
        authorization_prefix: str = NWON_DJANGO_SETTINGS.authorization_prefix,
    ):
        self.api_client.credentials(
            HTTP_AUTHORIZATION=f"{authorization_prefix} {token}"
        )

    def get(self, url: str) -> Response:
        return self.api_client.get(url)

    def post(self, url: str, body: AnyDict, body_format: RequestBodyFormat) -> Response:
        return self.api_client.post(url, body, format=body_format.value)

    def put(self, url: str, body: AnyDict, body_format: RequestBodyFormat) -> Response:
        return self.api_client.put(url, body, format=body_format.value)

    def patch(
        self, url: str, body: AnyDict, body_format: RequestBodyFormat
    ) -> Response:
        return self.api_client.patch(url, body, format=body_format.value)

    def delete(self, url: str) -> Response:
        return self.api_client.delete(url)



import sys
from _typeshed import StrPath
from collections.abc import Sequence
from typing import IO

__all__ = [
    "knownfiles",
    "inited",
    "MimeTypes",
    "guess_type",
    "guess_all_extensions",
    "guess_extension",
    "add_type",
    "init",
    "read_mime_types",
    "suffix_map",
    "encodings_map",
    "types_map",
    "common_types",
]

if sys.version_info >= (3, 8):
    def guess_type(url: StrPath, strict: bool = True) -> tuple[str | None, str | None]: ...

else:
    def guess_type(url: str, strict: bool = True) -> tuple[str | None, str | None]: ...

def guess_all_extensions(type: str, strict: bool = True) -> list[str]: ...
def guess_extension(type: str, strict: bool = True) -> str | None: ...
def init(files: Sequence[str] | None = None) -> None: ...
def read_mime_types(file: str) -> dict[str, str] | None: ...
def add_type(type: str, ext: str, strict: bool = True) -> None: ...

inited: bool
knownfiles: list[str]
suffix_map: dict[str, str]
encodings_map: dict[str, str]
types_map: dict[str, str]
common_types: dict[str, str]

class MimeTypes:
    suffix_map: dict[str, str]
    encodings_map: dict[str, str]
    types_map: tuple[dict[str, str], dict[str, str]]
    types_map_inv: tuple[dict[str, str], dict[str, str]]
    def __init__(self, filenames: tuple[str, ...] = (), strict: bool = True) -> None: ...
    def guess_extension(self, type: str, strict: bool = True) -> str | None: ...
    if sys.version_info >= (3, 8):
        def guess_type(self, url: StrPath, strict: bool = True) -> tuple[str | None, str | None]: ...
    else:
        def guess_type(self, url: str, strict: bool = True) -> tuple[str | None, str | None]: ...

    def guess_all_extensions(self, type: str, strict: bool = True) -> list[str]: ...
    def read(self, filename: str, strict: bool = True) -> None: ...
    def readfp(self, fp: IO[str], strict: bool = True) -> None: ...
    if sys.platform == "win32":
        def read_windows_registry(self, strict: bool = True) -> None: ...



from typing import Any

from django import template
from django.template.base import Parser, Token
from django.template.context import Context

register: Any

class AdminLogNode(template.Node):
    limit: str
    user: str
    varname: str
    def __init__(self, limit: str, varname: str, user: str | None) -> None: ...
    def render(self, context: Context) -> str: ...

def get_admin_log(parser: Parser, token: Token) -> AdminLogNode: ...



from django.contrib.sites.models import Site
from django.db import models

class FlatPage(models.Model):
    url: models.CharField
    title: models.CharField
    content: models.TextField
    enable_comments: models.BooleanField
    template_name: models.CharField
    registration_required: models.BooleanField
    sites: models.ManyToManyField[Site, Site]
    def get_absolute_url(self) -> str: ...



from nwon_django_toolbox.tests.api_method_tests.check_delete import (
    check_delete_basics,
    check_delete_not_allowed,
)
from nwon_django_toolbox.tests.api_method_tests.check_get import check_get_basics
from nwon_django_toolbox.tests.api_method_tests.check_patch import (
    check_patch_basics,
    check_patch_method_not_allowed,
    check_patch_parameters_failing,
    check_patch_parameters_successful,
    check_patch_read_only_field,
)
from nwon_django_toolbox.tests.api_method_tests.check_permissions import (
    check_delete_permissions,
    check_get_detail_permissions,
    check_get_list_permissions,
    check_patch_permissions,
    check_permission,
    check_permissions,
    check_post_permissions,
    check_put_permissions,
)
from nwon_django_toolbox.tests.api_method_tests.check_post import (
    check_post_basics,
    check_post_not_allowed,
    check_post_parameters_failing,
    check_post_parameters_not_required,
    check_post_parameters_required,
    check_post_parameters_successful,
    check_post_read_only_field,
)
from nwon_django_toolbox.tests.api_method_tests.check_put import (
    check_put_basics,
    check_put_not_allowed,
    check_put_parameters_failing,
    check_put_parameters_successful,
    check_put_read_only_field,
)
from nwon_django_toolbox.tests.helper.ensure_key_with_object_list import (
    ensure_key_with_object_list,
)
from nwon_django_toolbox.tests.helper.ensure_paged_results import ensure_paged_results

__all__ = [
    "check_delete_basics",
    "check_delete_not_allowed",
    "check_get_basics",
    "check_patch_basics",
    "check_patch_method_not_allowed",
    "check_patch_parameters_failing",
    "check_patch_parameters_successful",
    "check_patch_read_only_field",
    "check_put_basics",
    "check_put_not_allowed",
    "check_post_parameters_required",
    "check_put_parameters_failing",
    "check_put_parameters_successful",
    "check_put_read_only_field",
    "check_post_basics",
    "check_post_not_allowed",
    "check_post_parameters_failing",
    "check_post_parameters_successful",
    "check_post_read_only_field",
    "check_post_parameters_not_required",
    "ensure_key_with_object_list",
    "ensure_paged_results",
    "check_delete_permissions",
    "check_get_detail_permissions",
    "check_get_list_permissions",
    "check_patch_permissions",
    "check_permission",
    "check_permissions",
    "check_post_permissions",
    "check_put_permissions",
]



import sys
from tracemalloc import _FrameTupleT, _TraceTupleT
from typing import Optional, Sequence, Tuple

def _get_object_traceback(__obj: object) -> Optional[Sequence[_FrameTupleT]]: ...
def _get_traces() -> Sequence[_TraceTupleT]: ...
def clear_traces() -> None: ...
def get_traceback_limit() -> int: ...
def get_traced_memory() -> Tuple[int, int]: ...
def get_tracemalloc_memory() -> int: ...
def is_tracing() -> bool: ...

if sys.version_info >= (3, 9):
    def reset_peak() -> None: ...

def start(__nframe: int = ...) -> None: ...
def stop() -> None: ...



from collections.abc import Callable
from typing import TypeVar

from django.http.request import HttpRequest
from django.utils.functional import SimpleLazyObject

_R = TypeVar("_R", bound=HttpRequest)

def csrf(request: HttpRequest) -> dict[str, SimpleLazyObject]: ...
def debug(request: HttpRequest) -> dict[str, Callable | bool]: ...
def i18n(request: HttpRequest) -> dict[str, list[tuple[str, str]] | bool | str]: ...
def tz(request: HttpRequest) -> dict[str, str]: ...
def static(request: HttpRequest) -> dict[str, str]: ...
def media(request: HttpRequest) -> dict[str, str]: ...
def request(request: _R) -> dict[str, _R]: ...



from _typeshed import Incomplete, SupportsRead
from collections.abc import Mapping, Sequence
from typing import IO, Any

from .events import Event
from .nodes import Node
from .tokens import Token

def get_version_string() -> str: ...
def get_version() -> tuple[int, int, int]: ...

class Mark:
    name: Any
    index: int
    line: int
    column: int
    buffer: Any
    pointer: Any
    def __init__(self, name, index: int, line: int, column: int, buffer, pointer) -> None: ...
    def get_snippet(self): ...

class CParser:
    def __init__(self, stream: str | bytes | SupportsRead[str | bytes]) -> None: ...
    def dispose(self) -> None: ...
    def get_token(self) -> Token | None: ...
    def peek_token(self) -> Token | None: ...
    def check_token(self, *choices) -> bool: ...
    def get_event(self) -> Event | None: ...
    def peek_event(self) -> Event | None: ...
    def check_event(self, *choices) -> bool: ...
    def check_node(self) -> bool: ...
    def get_node(self) -> Node | None: ...
    def get_single_node(self) -> Node | None: ...
    def raw_parse(self) -> int: ...
    def raw_scan(self) -> int: ...

class CEmitter:
    def __init__(
        self,
        stream: IO[Any],
        canonical: Incomplete | None = ...,
        indent: int | None = ...,
        width: int | None = ...,
        allow_unicode: Incomplete | None = ...,
        line_break: str | None = ...,
        encoding: str | None = ...,
        explicit_start: Incomplete | None = ...,
        explicit_end: Incomplete | None = ...,
        version: Sequence[int] | None = ...,
        tags: Mapping[str, str] | None = ...,
    ) -> None: ...
    def dispose(self) -> None: ...
    def emit(self, event_object) -> None: ...
    def open(self) -> None: ...
    def close(self) -> None: ...
    def serialize(self, node) -> None: ...



from collections.abc import Iterable, Mapping
from typing import Any, Generic, TypeVar

from _typeshed import Self
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.expressions import Expression, Func
from django.db.models.query_utils import RegisterLookupMixin
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType, _ParamT
from django.utils.datastructures import OrderedSet
from typing_extensions import Literal

_T = TypeVar("_T")

class Lookup(Generic[_T]):
    lookup_name: str
    prepare_rhs: bool
    can_use_none_as_rhs: bool
    lhs: Any
    rhs: Any
    bilateral_transforms: list[type[Transform]]
    def __init__(self, lhs: Any, rhs: Any) -> None: ...
    def apply_bilateral_transforms(self, value: Expression) -> Expression: ...
    def batch_process_rhs(
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, rhs: OrderedSet | None = ...
    ) -> tuple[list[str], list[str]]: ...
    def get_source_expressions(self) -> list[Expression]: ...
    def set_source_expressions(self, new_exprs: list[Expression]) -> None: ...
    def get_prep_lookup(self) -> Any: ...
    def get_db_prep_lookup(self, value: _ParamT, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    def process_lhs(
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, lhs: Expression | None = ...
    ) -> _AsSqlType: ...
    def process_rhs(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    def rhs_is_direct_value(self) -> bool: ...
    def relabeled_clone(self: Self, relabels: Mapping[str, str]) -> Self: ...
    def get_group_by_cols(self, alias: str | None = ...) -> list[Expression]: ...
    def as_sql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    @property
    def contains_aggregate(self) -> bool: ...
    @property
    def contains_over_clause(self) -> bool: ...
    @property
    def is_summary(self) -> bool: ...
    @property
    def identity(self) -> tuple[type[Lookup], Any, Any]: ...

class Transform(RegisterLookupMixin, Func):
    bilateral: bool
    @property
    def lhs(self) -> Expression: ...
    def get_bilateral_transforms(self) -> list[type[Transform]]: ...

class BuiltinLookup(Lookup[_T]):
    def process_lhs(
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, lhs: Expression | None = ...
    ) -> _AsSqlType: ...
    def get_rhs_op(self, connection: BaseDatabaseWrapper, rhs: str) -> str: ...

class FieldGetDbPrepValueMixin:
    get_db_prep_lookup_value_is_iterable: bool
    def get_db_prep_lookup(self, value: _ParamT, connection: BaseDatabaseWrapper) -> _AsSqlType: ...

class FieldGetDbPrepValueIterableMixin(FieldGetDbPrepValueMixin):
    get_db_prep_lookup_value_is_iterable: Literal[True]
    def get_prep_lookup(self) -> Iterable[Any]: ...
    def resolve_expression_parameter(
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, sql: str, param: Any
    ) -> _AsSqlType: ...

class PostgresOperatorLookup(FieldGetDbPrepValueMixin, Lookup[_T]):
    postgres_operator: str
    def as_postgresql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...

class Exact(FieldGetDbPrepValueMixin, BuiltinLookup[_T]): ...
class IExact(BuiltinLookup[_T]): ...
class GreaterThan(FieldGetDbPrepValueMixin, BuiltinLookup[_T]): ...
class GreaterThanOrEqual(FieldGetDbPrepValueMixin, BuiltinLookup[_T]): ...
class LessThan(FieldGetDbPrepValueMixin, BuiltinLookup[_T]): ...
class LessThanOrEqual(FieldGetDbPrepValueMixin, BuiltinLookup[_T]): ...

class IntegerFieldFloatRounding:
    rhs: Any
    def get_prep_lookup(self) -> Any: ...

class IntegerGreaterThanOrEqual(IntegerFieldFloatRounding, GreaterThanOrEqual[int | float]): ...
class IntegerLessThan(IntegerFieldFloatRounding, LessThan[int | float]): ...

class In(FieldGetDbPrepValueIterableMixin, BuiltinLookup):
    def split_parameter_list_as_sql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> Any: ...

class PatternLookup(BuiltinLookup[str]):
    param_pattern: str

class Contains(PatternLookup): ...
class IContains(Contains): ...
class StartsWith(PatternLookup): ...
class IStartsWith(StartsWith): ...
class EndsWith(PatternLookup): ...
class IEndsWith(EndsWith): ...
class Range(FieldGetDbPrepValueIterableMixin, BuiltinLookup[_T]): ...
class IsNull(BuiltinLookup[bool]): ...
class Regex(BuiltinLookup[str]): ...
class IRegex(Regex): ...

class YearLookup(Lookup):
    def year_lookup_bounds(self, connection: BaseDatabaseWrapper, year: int) -> list[str]: ...
    def get_direct_rhs_sql(self, connection: BaseDatabaseWrapper, rhs: str) -> str: ...
    def get_bound_params(self, start: Any, finish: Any) -> Any: ...

class YearExact(YearLookup, Exact[_T]):
    def get_bound_params(self, start: Any, finish: Any) -> tuple[Any, Any]: ...

class YearGt(YearLookup, GreaterThan[_T]):
    def get_bound_params(self, start: Any, finish: Any) -> tuple[Any]: ...

class YearGte(YearLookup, GreaterThanOrEqual[_T]):
    def get_bound_params(self, start: Any, finish: Any) -> tuple[Any]: ...

class YearLt(YearLookup, LessThan[_T]):
    def get_bound_params(self, start: Any, finish: Any) -> tuple[Any]: ...

class YearLte(YearLookup, LessThanOrEqual[_T]):
    def get_bound_params(self, start: Any, finish: Any) -> tuple[Any]: ...

class UUIDTextMixin:
    rhs: Any
    def process_rhs(self, qn: Any, connection: BaseDatabaseWrapper) -> Any: ...

class UUIDIExact(UUIDTextMixin, IExact[_T]): ...
class UUIDContains(UUIDTextMixin, Contains): ...
class UUIDIContains(UUIDTextMixin, IContains): ...
class UUIDStartsWith(UUIDTextMixin, StartsWith): ...
class UUIDIStartsWith(UUIDTextMixin, IStartsWith): ...
class UUIDEndsWith(UUIDTextMixin, EndsWith): ...
class UUIDIEndsWith(UUIDTextMixin, IEndsWith): ...



from collections.abc import Callable, Sequence
from typing import Any

from rest_framework.renderers import BaseRenderer
from rest_framework.urlpatterns import _AnyURL

from . import coreapi as coreapi
from . import openapi as openapi
from .coreapi import AutoSchema as AutoSchema
from .coreapi import ManualSchema as ManualSchema
from .coreapi import SchemaGenerator as SchemaGenerator
from .generators import BaseSchemaGenerator
from .inspectors import DefaultSchema as DefaultSchema

def get_schema_view(
    title: str | None = ...,
    url: str | None = ...,
    description: str | None = ...,
    urlconf: str | None = ...,
    renderer_classes: Sequence[type[BaseRenderer]] | None = ...,
    public: bool = ...,
    patterns: Sequence[_AnyURL] | None = ...,
    generator_class: type[BaseSchemaGenerator] = ...,
    authentication_classes: Sequence[str] = ...,
    permission_classes: Sequence[str] = ...,
    version: str | None = ...,
) -> Callable[..., Any]: ...



from collections.abc import Sequence
from types import TracebackType
from typing import Any

from _typeshed import Self
from django.core.mail.message import EmailMessage

class BaseEmailBackend:
    fail_silently: bool
    def __init__(self, fail_silently: bool = ..., **kwargs: Any) -> None: ...
    def open(self) -> bool | None: ...
    def close(self) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...
    def send_messages(self, email_messages: Sequence[EmailMessage]) -> int: ...



from collections.abc import Sequence
from pathlib import Path
from typing import Any

from django.contrib.gis.geos import Point

GEOIP_SETTINGS: dict[str, Any]

class GeoIP2Exception(Exception): ...

class GeoIP2:
    MODE_AUTO: int
    MODE_MMAP_EXT: int
    MODE_MMAP: int
    MODE_FILE: int
    MODE_MEMORY: int
    cache_options: frozenset[int]
    def __init__(
        self, path: Path | str | None = ..., cache: int = ..., country: Any | None = ..., city: Any | None = ...
    ) -> None: ...
    def __del__(self) -> None: ...
    def city(self, query: str) -> dict[str, Any]: ...
    def country_code(self, query: str) -> str: ...
    def country_name(self, query: str) -> str: ...
    def country(self, query: str) -> dict[str, Any]: ...
    def coords(self, query: str, ordering: Sequence[str] = ...) -> tuple[float, float] | tuple[None, None]: ...
    def lon_lat(self, query: str) -> tuple[float, float] | tuple[None, None]: ...
    def lat_lon(self, query: str) -> tuple[float, float] | tuple[None, None]: ...
    def geos(self, query: str) -> Point: ...
    @property
    def info(self) -> str: ...
    @classmethod
    def open(cls, full_path: Path | str | None, cache: int) -> Any: ...



from collections.abc import Callable, Sequence
from typing import Any

from rest_framework.renderers import BaseRenderer
from rest_framework.schemas import SchemaGenerator
from rest_framework.urlpatterns import _AnyURL

def get_docs_view(
    title: str | None = ...,
    url: str | None = ...,
    description: str | None = ...,
    urlconf: str | None = ...,
    renderer_classes: Sequence[type[BaseRenderer]] | None = ...,
    public: bool = ...,
    patterns: Sequence[_AnyURL] | None = ...,
    generator_class: type[SchemaGenerator] = ...,
    authentication_classes: Sequence[str] = ...,
    permission_classes: Sequence[str] = ...,
) -> Callable[..., Any]: ...
def get_schemajs_view(
    title: str | None = ...,
    url: str | None = ...,
    description: str | None = ...,
    urlconf: str | None = ...,
    renderer_classes: Sequence[type[BaseRenderer]] | None = ...,
    public: bool = ...,
    patterns: Sequence[_AnyURL] | None = ...,
    generator_class: type[SchemaGenerator] = ...,
    authentication_classes: Sequence[str] = ...,
    permission_classes: Sequence[str] = ...,
) -> Callable[..., Any]: ...
def include_docs_urls(
    title: str | None = ...,
    url: str | None = ...,
    description: str | None = ...,
    urlconf: str | None = ...,
    renderer_classes: Sequence[type[BaseRenderer]] | None = ...,
    public: bool = ...,
    patterns: Sequence[_AnyURL] | None = ...,
    generator_class: type[SchemaGenerator] = ...,
    authentication_classes: Sequence[str] = ...,
    permission_classes: Sequence[str] = ...,
) -> Any: ...



from typing import Any

from django.http.response import HttpResponse

def compress_kml(kml: str) -> bytes: ...
def render_to_kml(*args: Any, **kwargs: Any) -> HttpResponse: ...
def render_to_kmz(*args: Any, **kwargs: Any) -> HttpResponse: ...



import sys
from typing import Text

# This module is only available on Windows
if sys.platform == "win32":
    LK_LOCK: int
    LK_NBLCK: int
    LK_NBRLCK: int
    LK_RLCK: int
    LK_UNLCK: int
    def locking(__fd: int, __mode: int, __nbytes: int) -> None: ...
    def setmode(__fd: int, __mode: int) -> int: ...
    def open_osfhandle(__handle: int, __flags: int) -> int: ...
    def get_osfhandle(__fd: int) -> int: ...
    def kbhit() -> bool: ...
    def getch() -> bytes: ...
    def getwch() -> Text: ...
    def getche() -> bytes: ...
    def getwche() -> Text: ...
    def putch(__char: bytes) -> None: ...
    def putwch(__unicode_char: Text) -> None: ...
    def ungetch(__char: bytes) -> None: ...
    def ungetwch(__unicode_char: Text) -> None: ...
    def heapmin() -> None: ...



from _typeshed import Incomplete
from typing import Any

class _Operand:
    im: Any
    def __init__(self, im) -> None: ...
    def apply(self, op, im1, im2: Incomplete | None = None, mode: Incomplete | None = None): ...
    def __bool__(self) -> bool: ...
    def __abs__(self): ...
    def __pos__(self): ...
    def __neg__(self): ...
    def __add__(self, other): ...
    def __radd__(self, other): ...
    def __sub__(self, other): ...
    def __rsub__(self, other): ...
    def __mul__(self, other): ...
    def __rmul__(self, other): ...
    def __truediv__(self, other): ...
    def __rtruediv__(self, other): ...
    def __mod__(self, other): ...
    def __rmod__(self, other): ...
    def __pow__(self, other): ...
    def __rpow__(self, other): ...
    def __invert__(self): ...
    def __and__(self, other): ...
    def __rand__(self, other): ...
    def __or__(self, other): ...
    def __ror__(self, other): ...
    def __xor__(self, other): ...
    def __rxor__(self, other): ...
    def __lshift__(self, other): ...
    def __rshift__(self, other): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

def imagemath_int(self): ...
def imagemath_float(self): ...
def imagemath_equal(self, other): ...
def imagemath_notequal(self, other): ...
def imagemath_min(self, other): ...
def imagemath_max(self, other): ...
def imagemath_convert(self, mode): ...

ops: Any

def eval(expression, _dict={}, **kw): ...



from collections.abc import Iterable, Sequence
from typing import Any

from django.contrib.syndication.views import Feed as BaseFeed
from django.utils.feedgenerator import Atom1Feed, Rss201rev2Feed
from django.utils.xmlutils import SimplerXMLGenerator

class GeoFeedMixin:
    def georss_coords(self, coords: Iterable[Sequence[float]]) -> str: ...
    def add_georss_point(self, handler: SimplerXMLGenerator, coords: Sequence[float], w3c_geo: bool = ...) -> None: ...
    def add_georss_element(self, handler: SimplerXMLGenerator, item: dict[str, Any], w3c_geo: bool = ...) -> None: ...

class GeoRSSFeed(Rss201rev2Feed, GeoFeedMixin):
    def rss_attributes(self) -> dict[str, str]: ...

class GeoAtom1Feed(Atom1Feed, GeoFeedMixin): ...

class W3CGeoFeed(Rss201rev2Feed, GeoFeedMixin):
    def rss_attributes(self) -> dict[str, str]: ...

class Feed(BaseFeed): ...



import sys
from typing import Iterable, Optional, SupportsFloat, SupportsInt, Tuple, overload

e: float
pi: float
if sys.version_info >= (3, 5):
    inf: float
    nan: float
if sys.version_info >= (3, 6):
    tau: float

def acos(__x: SupportsFloat) -> float: ...
def acosh(__x: SupportsFloat) -> float: ...
def asin(__x: SupportsFloat) -> float: ...
def asinh(__x: SupportsFloat) -> float: ...
def atan(__x: SupportsFloat) -> float: ...
def atan2(__y: SupportsFloat, __x: SupportsFloat) -> float: ...
def atanh(__x: SupportsFloat) -> float: ...

if sys.version_info >= (3,):
    def ceil(__x: SupportsFloat) -> int: ...

else:
    def ceil(__x: SupportsFloat) -> float: ...

if sys.version_info >= (3, 8):
    def comb(__n: int, __k: int) -> int: ...

def copysign(__x: SupportsFloat, __y: SupportsFloat) -> float: ...
def cos(__x: SupportsFloat) -> float: ...
def cosh(__x: SupportsFloat) -> float: ...
def degrees(__x: SupportsFloat) -> float: ...

if sys.version_info >= (3, 8):
    def dist(__p: Iterable[SupportsFloat], __q: Iterable[SupportsFloat]) -> float: ...

def erf(__x: SupportsFloat) -> float: ...
def erfc(__x: SupportsFloat) -> float: ...
def exp(__x: SupportsFloat) -> float: ...
def expm1(__x: SupportsFloat) -> float: ...
def fabs(__x: SupportsFloat) -> float: ...
def factorial(__x: SupportsInt) -> int: ...

if sys.version_info >= (3,):
    def floor(__x: SupportsFloat) -> int: ...

else:
    def floor(__x: SupportsFloat) -> float: ...

def fmod(__x: SupportsFloat, __y: SupportsFloat) -> float: ...
def frexp(__x: SupportsFloat) -> Tuple[float, int]: ...
def fsum(__seq: Iterable[float]) -> float: ...
def gamma(__x: SupportsFloat) -> float: ...

if sys.version_info >= (3, 9):
    def gcd(*integers: int) -> int: ...

elif sys.version_info >= (3, 5):
    def gcd(__x: int, __y: int) -> int: ...

if sys.version_info >= (3, 8):
    def hypot(*coordinates: SupportsFloat) -> float: ...

else:
    def hypot(__x: SupportsFloat, __y: SupportsFloat) -> float: ...

if sys.version_info >= (3, 5):
    def isclose(a: SupportsFloat, b: SupportsFloat, *, rel_tol: SupportsFloat = ..., abs_tol: SupportsFloat = ...) -> bool: ...

def isinf(__x: SupportsFloat) -> bool: ...

if sys.version_info >= (3,):
    def isfinite(__x: SupportsFloat) -> bool: ...

def isnan(__x: SupportsFloat) -> bool: ...

if sys.version_info >= (3, 8):
    def isqrt(__n: int) -> int: ...

if sys.version_info >= (3, 9):
    def lcm(*integers: int) -> int: ...

def ldexp(__x: SupportsFloat, __i: int) -> float: ...
def lgamma(__x: SupportsFloat) -> float: ...
def log(x: SupportsFloat, base: SupportsFloat = ...) -> float: ...
def log10(__x: SupportsFloat) -> float: ...
def log1p(__x: SupportsFloat) -> float: ...

if sys.version_info >= (3, 3):
    def log2(__x: SupportsFloat) -> float: ...

def modf(__x: SupportsFloat) -> Tuple[float, float]: ...

if sys.version_info >= (3, 9):
    def nextafter(__x: SupportsFloat, __y: SupportsFloat) -> float: ...

if sys.version_info >= (3, 8):
    def perm(__n: int, __k: Optional[int] = ...) -> int: ...

def pow(__x: SupportsFloat, __y: SupportsFloat) -> float: ...

if sys.version_info >= (3, 8):
    @overload
    def prod(__iterable: Iterable[int], *, start: int = ...) -> int: ...  # type: ignore
    @overload
    def prod(__iterable: Iterable[SupportsFloat], *, start: SupportsFloat = ...) -> float: ...

def radians(__x: SupportsFloat) -> float: ...

if sys.version_info >= (3, 7):
    def remainder(__x: SupportsFloat, __y: SupportsFloat) -> float: ...

def sin(__x: SupportsFloat) -> float: ...
def sinh(__x: SupportsFloat) -> float: ...
def sqrt(__x: SupportsFloat) -> float: ...
def tan(__x: SupportsFloat) -> float: ...
def tanh(__x: SupportsFloat) -> float: ...
def trunc(__x: SupportsFloat) -> int: ...

if sys.version_info >= (3, 9):
    def ulp(__x: SupportsFloat) -> float: ...



from typing import List, Optional, Tuple

class Extension:
    def __init__(
        self,
        name: str,
        sources: List[str],
        include_dirs: Optional[List[str]] = ...,
        define_macros: Optional[List[Tuple[str, Optional[str]]]] = ...,
        undef_macros: Optional[List[str]] = ...,
        library_dirs: Optional[List[str]] = ...,
        libraries: Optional[List[str]] = ...,
        runtime_library_dirs: Optional[List[str]] = ...,
        extra_objects: Optional[List[str]] = ...,
        extra_compile_args: Optional[List[str]] = ...,
        extra_link_args: Optional[List[str]] = ...,
        export_symbols: Optional[List[str]] = ...,
        swig_opts: Optional[str] = ...,  # undocumented
        depends: Optional[List[str]] = ...,
        language: Optional[str] = ...,
        optional: Optional[bool] = ...,
    ) -> None: ...



from collections.abc import Iterator
from typing import IO, Any

from django.core.serializers.base import DeserializedObject
from django.core.serializers.python import Serializer as PythonSerializer

class Serializer(PythonSerializer):
    json_kwargs: dict[str, Any]

def Deserializer(
    stream_or_string: IO[bytes] | IO[str] | bytes | str, **options: Any
) -> Iterator[DeserializedObject]: ...



from typing import TypeVar

from django.contrib.sessions.base_session import AbstractBaseSession, BaseSessionManager

_T = TypeVar("_T", bound=Session)

class SessionManager(BaseSessionManager[_T]): ...
class Session(AbstractBaseSession): ...



from django.contrib.admin import HORIZONTAL as HORIZONTAL
from django.contrib.admin import VERTICAL as VERTICAL
from django.contrib.admin import AdminSite as AdminSite
from django.contrib.admin import ModelAdmin as ModelAdmin
from django.contrib.admin import StackedInline as StackedInline
from django.contrib.admin import TabularInline as TabularInline
from django.contrib.admin import action as action
from django.contrib.admin import autodiscover as autodiscover
from django.contrib.admin import register as register
from django.contrib.admin import site as site
from django.contrib.gis.admin.options import GeoModelAdmin as GeoModelAdmin
from django.contrib.gis.admin.options import GISModelAdmin as GISModelAdmin
from django.contrib.gis.admin.options import OSMGeoAdmin as OSMGeoAdmin
from django.contrib.gis.admin.widgets import OpenLayersWidget as OpenLayersWidget



from typing import Optional

from django.db.models.base import Model

from nwon_django_toolbox.tests.api_client.api_test import ApiTest
from nwon_django_toolbox.url_helper import detail_url_for_model, list_url_for_model


def check_delete_basics(
    model: Model,
    authentication_token: Optional[str] = None,
):
    not_found_url = detail_url_for_model(model, "non-existing-id")
    detail_url = detail_url_for_model(model)

    api_test = ApiTest()

    api_test.delete_unauthorized(not_found_url)
    api_test.delete_unauthorized(detail_url)

    if authentication_token:
        api_test.set_bearer_token(authentication_token)
        api_test.delete_not_found(not_found_url)
        api_test.delete_successful(detail_url)
        api_test.get_not_found(detail_url)


def check_delete_not_allowed(
    model: Model,
    authentication_token: Optional[str] = None,
):
    api_test = ApiTest(token=authentication_token)

    detail_url = detail_url_for_model(model)
    list_url = list_url_for_model(model)

    api_test.delete_method_not_allowed(detail_url)
    api_test.delete_method_not_allowed(list_url)


__all__ = [
    "check_delete_basics",
    "check_delete_not_allowed",
]



from typing import Any, Generic, TypeVar

from django.db import models
from django.http import HttpRequest, HttpResponse
from django.views.generic.base import ContextMixin, TemplateResponseMixin, View

_M = TypeVar("_M", bound=models.Model)

class SingleObjectMixin(Generic[_M], ContextMixin):
    model: type[_M]
    queryset: models.query.QuerySet[_M] | None
    slug_field: str
    context_object_name: str | None
    slug_url_kwarg: str
    pk_url_kwarg: str
    query_pk_and_slug: bool
    def get_object(self, queryset: models.query.QuerySet[_M] | None = ...) -> _M: ...
    def get_queryset(self) -> models.query.QuerySet[_M]: ...
    def get_slug_field(self) -> str: ...
    def get_context_object_name(self, obj: _M) -> str | None: ...

class BaseDetailView(SingleObjectMixin[_M], View):
    object: _M
    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...

class SingleObjectTemplateResponseMixin(TemplateResponseMixin):
    template_name_field: str | None
    template_name_suffix: str

class DetailView(SingleObjectTemplateResponseMixin, BaseDetailView[_M]): ...



from collections.abc import Callable, Sequence
from io import BytesIO
from typing import Any

from django.contrib.sessions.backends.base import SessionBase
from django.core.handlers import base
from django.http import HttpRequest
from django.http.response import HttpResponseBase
from typing_extensions import TypeAlias

_WSGIEnviron: TypeAlias = dict[str, Any]

class LimitedStream:
    stream: BytesIO
    remaining: int
    buffer: bytes
    buf_size: int
    def __init__(self, stream: BytesIO, limit: int, buf_size: int = ...) -> None: ...
    def read(self, size: int | None = ...) -> bytes: ...
    def readline(self, size: int | None = ...) -> bytes: ...

class WSGIRequest(HttpRequest):
    environ: _WSGIEnviron
    session: SessionBase
    encoding: Any
    def __init__(self, environ: _WSGIEnviron) -> None: ...

class WSGIHandler(base.BaseHandler):
    request_class: type[WSGIRequest]
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __call__(
        self,
        environ: _WSGIEnviron,
        start_response: Callable[[str, Sequence[tuple[str, str]]], None],
    ) -> HttpResponseBase: ...

def get_path_info(environ: _WSGIEnviron) -> str: ...
def get_script_name(environ: _WSGIEnviron) -> str: ...
def get_bytes_from_wsgi(environ: _WSGIEnviron, key: str, default: str) -> bytes: ...
def get_str_from_wsgi(environ: _WSGIEnviron, key: str, default: str) -> str: ...



from django.contrib.sites.models import Site
from django.contrib.sites.requests import RequestSite
from django.http.request import HttpRequest

def get_current_site(request: HttpRequest | None) -> Site | RequestSite: ...



from typing import Any

from django.http import FileResponse, HttpResponse
from django.http.request import HttpRequest

def serve(
    request: HttpRequest, path: str, document_root: str | None = ..., show_indexes: bool = ...
) -> FileResponse: ...

DEFAULT_DIRECTORY_INDEX_TEMPLATE: str
template_translatable: Any

def directory_index(path: Any, fullpath: Any) -> HttpResponse: ...
def was_modified_since(header: str | None = ..., mtime: float = ..., size: int = ...) -> bool: ...



from django.http import HttpRequest, HttpResponse

def kml(
    request: HttpRequest,
    label: str,
    model: str,
    field_name: str | None = ...,
    compress: bool = ...,
    using: str = ...,
) -> HttpResponse: ...
def kmz(
    request: HttpRequest, label: str, model: str, field_name: str | None = ..., using: str = ...
) -> HttpResponse: ...



from enum import Enum
from typing import Any, List, Tuple


class DjangoReadyEnum(Enum):
    @classmethod
    def choices(cls) -> List[Tuple[Any, str]]:
        return [(key.value, key.name) for key in cls]


__all__ = ["DjangoReadyEnum"]



from collections.abc import Iterable, Sequence
from typing import Any, Generic, TypeVar, Union, overload  # noqa: Y037   # https://github.com/python/mypy/issues/12211

from django.apps.config import AppConfig
from django.apps.registry import Apps
from django.contrib.contenttypes.fields import GenericForeignKey
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.base import Model
from django.db.models.constraints import BaseConstraint, UniqueConstraint
from django.db.models.fields import AutoField, Field
from django.db.models.fields.related import ManyToManyField, OneToOneField
from django.db.models.fields.reverse_related import ForeignObjectRel
from django.db.models.manager import Manager
from django.db.models.query_utils import PathInfo
from django.utils.datastructures import ImmutableList, _ListOrTuple
from django.utils.functional import _StrOrPromise
from typing_extensions import Literal, TypeAlias

PROXY_PARENTS: object
EMPTY_RELATION_TREE: Any
IMMUTABLE_WARNING: str
DEFAULT_NAMES: tuple[str, ...]

# https://github.com/python/mypy/issues/12211
_OptionTogetherT: TypeAlias = Union[_ListOrTuple[_ListOrTuple[str] | str], set[tuple[str, ...]]]  # noqa: Y047

@overload
def normalize_together(option_together: _ListOrTuple[_ListOrTuple[str] | str]) -> tuple[tuple[str, ...], ...]: ...

# Any other value will be returned unchanged, but probably only set is semantically allowed
@overload
def normalize_together(option_together: set[tuple[str, ...]]) -> set[tuple[str, ...]]: ...

_T = TypeVar("_T")

def make_immutable_fields_list(name: str, data: Iterable[_T]) -> ImmutableList[_T]: ...

_M = TypeVar("_M", bound=Model)

class Options(Generic[_M]):
    constraints: list[BaseConstraint]
    FORWARD_PROPERTIES: set[str]
    REVERSE_PROPERTIES: set[str]
    default_apps: Any
    local_fields: list[Field]
    local_many_to_many: list[ManyToManyField]
    private_fields: list[Any]
    local_managers: list[Manager]
    base_manager_name: str | None
    default_manager_name: str | None
    model_name: str | None
    verbose_name: _StrOrPromise | None
    verbose_name_plural: _StrOrPromise | None
    db_table: str
    ordering: Sequence[str] | None
    indexes: list[Any]
    unique_together: Sequence[tuple[str]]  # Are always normalized
    index_together: Sequence[tuple[str]]  # Are always normalized
    select_on_save: bool
    default_permissions: Sequence[str]
    permissions: list[Any]
    object_name: str | None
    app_label: str
    get_latest_by: Sequence[str] | None
    order_with_respect_to: str | None
    db_tablespace: str
    required_db_features: list[str]
    required_db_vendor: Literal["sqlite", "postgresql", "mysql", "oracle"] | None
    meta: type | None
    pk: Field | None
    auto_field: AutoField | None
    abstract: bool
    managed: bool
    proxy: bool
    proxy_for_model: type[Model] | None
    concrete_model: type[Model] | None
    swappable: str | None
    parents: dict[type[Model], GenericForeignKey | Field]
    auto_created: bool
    related_fkey_lookups: list[Any]
    apps: Apps
    default_related_name: str | None
    model: type[Model]
    original_attrs: dict[str, Any]
    def __init__(self, meta: type | None, app_label: str | None = ...) -> None: ...
    @property
    def label(self) -> str: ...
    @property
    def label_lower(self) -> str: ...
    @property
    def app_config(self) -> AppConfig: ...
    @property
    def installed(self) -> bool: ...
    def contribute_to_class(self, cls: type[Model], name: str) -> None: ...
    def add_manager(self, manager: Manager) -> None: ...
    def add_field(self, field: GenericForeignKey | Field[Any, Any], private: bool = ...) -> None: ...
    # if GenericForeignKey is passed as argument, it has primary_key = True set before
    def setup_pk(self, field: GenericForeignKey | Field[Any, Any]) -> None: ...
    def setup_proxy(self, target: type[Model]) -> None: ...
    def can_migrate(self, connection: BaseDatabaseWrapper | str) -> bool: ...
    @property
    def verbose_name_raw(self) -> str: ...
    @property
    def swapped(self) -> str | None: ...
    @property
    def fields_map(self) -> dict[str, Field[Any, Any] | ForeignObjectRel]: ...
    @property
    def managers(self) -> ImmutableList[Manager]: ...
    @property
    def managers_map(self) -> dict[str, Manager]: ...
    @property
    def base_manager(self) -> Manager: ...
    @property
    def default_manager(self) -> Manager | None: ...
    @property
    def fields(self) -> ImmutableList[Field[Any, Any]]: ...
    def get_field(self, field_name: str) -> Field | ForeignObjectRel | GenericForeignKey: ...
    def get_base_chain(self, model: type[Model]) -> list[type[Model]]: ...
    def get_parent_list(self) -> list[type[Model]]: ...
    def get_ancestor_link(self, ancestor: type[Model]) -> OneToOneField | None: ...
    def get_path_to_parent(self, parent: type[Model]) -> list[PathInfo]: ...
    def get_path_from_parent(self, parent: type[Model]) -> list[PathInfo]: ...
    def get_fields(
        self, include_parents: bool = ..., include_hidden: bool = ...
    ) -> list[Field[Any, Any] | ForeignObjectRel | GenericForeignKey]: ...
    @property
    def total_unique_constraints(self) -> list[UniqueConstraint]: ...
    @property
    def db_returning_fields(self) -> list[Field[Any, Any]]: ...



from typing import Any

from django.db.backends.mysql.introspection import DatabaseIntrospection

class MySQLIntrospection(DatabaseIntrospection):
    data_types_reverse: Any
    def get_geometry_type(self, table_name: Any, description: Any) -> Any: ...
    def supports_spatial_index(self, cursor: Any, table_name: Any) -> Any: ...



from django.core.management.base import BaseCommand
from django.db.migrations.loader import MigrationLoader
from django.db.migrations.migration import Migration

class Command(BaseCommand):
    verbosity: int
    interactive: bool
    def find_migration(self, loader: MigrationLoader, app_label: str, name: str) -> Migration: ...
